@@ -70,12 +70,14 @@ from hypothesis.executors import default_new_style_executor, new_style_executor
 from hypothesis.internal.compat import (
     PYPY,
     BaseExceptionGroup,
+    add_note,
     bad_django_TestCase,
     get_type_hints,
     int_from_bytes,
 )
 from hypothesis.internal.conjecture.data import ConjectureData, Status
 from hypothesis.internal.conjecture.engine import BUFFER_SIZE, ConjectureRunner
+from hypothesis.internal.conjecture.junkdrawer import ensure_free_stackframes
 from hypothesis.internal.conjecture.shrinker import sort_key
 from hypothesis.internal.entropy import deterministic_PRNG
 from hypothesis.internal.escalation import (
@@ -164,7 +166,7 @@ class example:
 
     def xfail(
         self,
-        condition: bool = True,
+        condition: bool = True,  # noqa: FBT002
         *,
         reason: str = "",
         raises: Union[
@@ -199,7 +201,7 @@ class example:
             )
         ):
             raise InvalidArgument(
-                f"raises={raises!r} must be an exception type or tuple of exception types"
+                f"{raises=} must be an exception type or tuple of exception types"
             )
         if condition:
             self._this_example = attr.evolve(
@@ -207,7 +209,7 @@ class example:
             )
         return self
 
-    def via(self, *whence: str) -> "example":
+    def via(self, whence: str, /) -> "example":
         """Attach a machine-readable label noting whence this example came.
 
         The idea is that tools will be able to add ``@example()`` cases for you, e.g.
@@ -242,26 +244,11 @@ class example:
                 pass
 
         """
-        if len(whence) != 1 or not isinstance(whence[0], str):
+        if not isinstance(whence, str):
             raise InvalidArgument(".via() must be passed a string")
         # This is deliberately a no-op at runtime; the tools operate on source code.
         return self
 
-    if sys.version_info[:2] >= (3, 8):
-        # We want a positional-only argument, and on Python 3.8+ we'll get it.
-        __sig = get_signature(via)
-        via = define_function_signature(
-            name=via.__name__,
-            docstring=via.__doc__,
-            signature=__sig.replace(
-                parameters=[
-                    p.replace(kind=inspect.Parameter.POSITIONAL_ONLY)
-                    for p in __sig.parameters.values()
-                ]
-            ),
-        )(via)
-        del __sig
-
 
 def seed(seed: Hashable) -> Callable[[TestFunc], TestFunc]:
     """seed: Start the test execution from a specific seed.
@@ -614,9 +601,10 @@ def process_arguments_to_given(wrapped_test, arguments, kwargs, given_kwargs, pa
 
     arguments = tuple(arguments)
 
-    for k, s in given_kwargs.items():
-        check_strategy(s, name=k)
-        s.validate()
+    with ensure_free_stackframes():
+        for k, s in given_kwargs.items():
+            check_strategy(s, name=k)
+            s.validate()
 
     stuff = Stuff(selfy=selfy, args=arguments, kwargs=kwargs, given_kwargs=given_kwargs)
 
@@ -707,6 +695,7 @@ class StateForActualGivenExecution:
     def execute_once(
         self,
         data,
+        *,
         print_example=False,
         is_final=False,
         expected_failure=None,
@@ -879,7 +868,8 @@ class StateForActualGivenExecution:
         except (
             HypothesisDeprecationWarning,
             FailedHealthCheck,
-        ) + skip_exceptions_to_reraise():
+            *skip_exceptions_to_reraise(),
+        ):
             # These are fatal errors or control exceptions that should stop the
             # engine, so we re-raise them.
             raise
@@ -1020,15 +1010,6 @@ class StateForActualGivenExecution:
         _raise_to_user(errors_to_report, self.settings, report_lines)
 
 
-def add_note(exc, note):
-    try:
-        exc.add_note(note)
-    except AttributeError:
-        if not hasattr(exc, "__notes__"):
-            exc.__notes__ = []
-        exc.__notes__.append(note)
-
-
 def _raise_to_user(errors_to_report, settings, target_lines, trailer=""):
     """Helper function for attaching notes and grouping multiple errors."""
     failing_prefix = "Falsifying example: "
@@ -1200,6 +1181,8 @@ def given(
                 )
             given_kwargs[name] = st.from_type(hints[name])
 
+        prev_self = Unset = object()
+
         @impersonate(test)
         @define_function_signature(test.__name__, test.__doc__, new_signature)
         def wrapped_test(*arguments, **kwargs):
@@ -1261,6 +1244,23 @@ def given(
                     "to ensure that each example is run in a separate "
                     "database transaction."
                 )
+            if settings.database is not None:
+                nonlocal prev_self
+                # Check selfy really is self (not e.g. a mock) before we health-check
+                cur_self = (
+                    stuff.selfy
+                    if getattr(type(stuff.selfy), test.__name__, None) is wrapped_test
+                    else None
+                )
+                if prev_self is Unset:
+                    prev_self = cur_self
+                elif cur_self is not prev_self:
+                    msg = (
+                        f"The method {test.__qualname__} was called from multiple "
+                        "different executors. This may lead to flaky tests and "
+                        "nonreproducible errors when replaying from database."
+                    )
+                    fail_health_check(settings, msg, HealthCheck.differing_executors)
 
             state = StateForActualGivenExecution(
                 test_runner, stuff, test, settings, random, wrapped_test
@@ -1490,6 +1490,9 @@ def find(
     )
 
     if database_key is None and settings.database is not None:
+        # Note: The database key is not guaranteed to be unique. If not, replaying
+        # of database examples may fail to reproduce due to being replayed on the
+        # wrong condition.
         database_key = function_digest(condition)
 
     if not isinstance(specifier, SearchStrategy):
@@ -1506,7 +1509,7 @@ def find(
     def test(v):
         if condition(v):
             last[:] = [v]
-            raise Found()
+            raise Found
 
     if random is not None:
         test = seed(random.getrandbits(64))(test)
