@@ -8,12 +8,8 @@
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
 
-import sys
-
-if sys.version_info[:2] < (3, 8):
-    raise RuntimeError("The Array API standard requires Python 3.8 or later")
-
 import math
+import sys
 from numbers import Real
 from types import SimpleNamespace
 from typing import (
@@ -69,10 +65,10 @@ __all__ = [
 ]
 
 
-RELEASED_VERSIONS = ("2021.12",)
-NOMINAL_VERSIONS = RELEASED_VERSIONS + ("draft",)
+RELEASED_VERSIONS = ("2021.12", "2022.12")
+NOMINAL_VERSIONS = (*RELEASED_VERSIONS, "draft")
 assert sorted(NOMINAL_VERSIONS) == list(NOMINAL_VERSIONS)  # sanity check
-NominalVersion = Literal["2021.12", "draft"]
+NominalVersion = Literal["2021.12", "2022.12", "draft"]
 assert get_args(NominalVersion) == NOMINAL_VERSIONS  # sanity check
 
 
@@ -83,7 +79,7 @@ FLOAT_NAMES = ("float32", "float64")
 REAL_NAMES = ALL_INT_NAMES + FLOAT_NAMES
 COMPLEX_NAMES = ("complex64", "complex128")
 NUMERIC_NAMES = REAL_NAMES + COMPLEX_NAMES
-DTYPE_NAMES = ("bool",) + NUMERIC_NAMES
+DTYPE_NAMES = ("bool", *NUMERIC_NAMES)
 
 DataType = TypeVar("DataType")
 
@@ -118,6 +114,7 @@ def warn_on_missing_dtypes(xp: Any, stubs: List[str]) -> None:
         f"Array module {xp.__name__} does not have the following "
         f"dtypes in its namespace: {f_stubs}",
         HypothesisWarning,
+        stacklevel=3,
     )
 
 
@@ -194,7 +191,7 @@ def _from_dtype(
     allow_subnormal: Optional[bool] = None,
     exclude_min: Optional[bool] = None,
     exclude_max: Optional[bool] = None,
-) -> st.SearchStrategy[Union[bool, int, float]]:
+) -> st.SearchStrategy[Union[bool, int, float, complex]]:
     """Return a strategy for any value of the given dtype.
 
     Values generated are of the Python scalar which is
@@ -297,31 +294,22 @@ def _from_dtype(
 
         return st.floats(width=finfo.bits, **kw)
     else:
-        # A less-inelegant solution to support complex dtypes exists, but as
-        # this is currently a draft feature, we might as well wait for
-        # discussion of complex inspection to resolve first - a better method
-        # might become available soon enough.
-        # See https://github.com/data-apis/array-api/issues/433
-        for attr in ["float32", "float64", "complex64"]:
-            if not hasattr(xp, attr):
-                raise NotImplementedError(
-                    f"Array module {xp.__name__} has no dtype {attr}, which is "
-                    "currently required for xps.from_dtype() to work with "
-                    "any complex dtype."
-                )
-        component_dtype = xp.float32 if dtype == xp.complex64 else xp.float64
-
-        floats = _from_dtype(
-            xp,
-            api_version,
-            component_dtype,
+        finfo = xp.finfo(dtype)
+        # See above comment on FTZ inference. We explicitly infer with a
+        # complex array, in case complex arrays have different FTZ behaviour
+        # than arrays of the respective composite float.
+        if allow_subnormal is None:
+            subnormal = next_down(finfo.smallest_normal, width=finfo.bits)
+            x = xp.asarray(complex(subnormal, subnormal), dtype=dtype)
+            builtin_x = complex(x)
+            allow_subnormal = builtin_x.real != 0 and builtin_x.imag != 0
+        return st.complex_numbers(
             allow_nan=allow_nan,
             allow_infinity=allow_infinity,
             allow_subnormal=allow_subnormal,
+            width=finfo.bits * 2,
         )
 
-        return st.builds(complex, floats, floats)  # type: ignore[arg-type]
-
 
 class ArrayStrategy(st.SearchStrategy):
     def __init__(
@@ -410,21 +398,8 @@ class ArrayStrategy(st.SearchStrategy):
             # our elements strategy to those indices.
 
             fill_val = data.draw(self.fill)
-            try:
-                result = self.xp.full(self.array_size, fill_val, dtype=self.dtype)
-            except Exception as e:
-                raise InvalidArgument(
-                    f"Could not create full array of dtype={self.dtype} "
-                    f"with fill value {fill_val!r}"
-                ) from e
-            sample = result[0]
-            self.check_set_value(fill_val, sample, self.fill)
-            if self.unique and not self.xp.all(self.xp.isnan(result)):
-                raise InvalidArgument(
-                    f"Array module {self.xp.__name__} did not recognise fill "
-                    f"value {fill_val!r} as NaN - instead got {sample!r}. "
-                    "Cannot fill unique array with non-NaN values."
-                )
+            result_obj = [fill_val for _ in range(self.array_size)]
+            fill_mask = [True for _ in range(self.array_size)]
 
             elements = cu.many(
                 data,
@@ -454,19 +429,29 @@ class ArrayStrategy(st.SearchStrategy):
                         continue
                     else:
                         seen.add(val)
-                try:
-                    result[i] = val
-                except Exception as e:
-                    raise InvalidArgument(
-                        f"Could not add generated array element {val!r} "
-                        f"of type {type(val)} to array of dtype {result.dtype}."
-                    ) from e
-                self.check_set_value(val, result[i], self.elements_strategy)
+                result_obj[i] = val
                 assigned.add(i)
+                fill_mask[i] = False
 
-        result = self.xp.reshape(result, self.shape)
+            try:
+                result = self.xp.asarray(result_obj, dtype=self.dtype)
+            except Exception as e:
+                f_expr = f"xp.asarray({result_obj}, dtype={self.dtype})"
+                raise InvalidArgument(f"Could not create array via {f_expr}") from e
 
-        return result
+            for i, val in enumerate(result_obj):
+                val_0d = result[i]
+                if fill_mask[i] and self.unique:
+                    if not self.xp.isnan(val_0d):
+                        raise InvalidArgument(
+                            f"Array module {self.xp.__name__} did not recognise fill "
+                            f"value {fill_val!r} as NaN - instead got {val_0d!r}. "
+                            "Cannot fill unique array with non-NaN values."
+                        )
+                else:
+                    self.check_set_value(val, val_0d, self.elements_strategy)
+
+        return self.xp.reshape(result, self.shape)
 
 
 def _arrays(
@@ -549,7 +534,7 @@ def _arrays(
     your tests to run in reasonable time.
     """
     check_xp_attributes(
-        xp, ["finfo", "asarray", "zeros", "full", "all", "isnan", "isfinite", "reshape"]
+        xp, ["finfo", "asarray", "zeros", "all", "isnan", "isfinite", "reshape"]
     )
 
     if isinstance(dtype, st.SearchStrategy):
@@ -573,7 +558,7 @@ def _arrays(
         raise InvalidArgument(f"shape={shape} is not a valid shape or strategy")
     check_argument(
         all(isinstance(x, int) and x >= 0 for x in shape),
-        f"shape={shape!r}, but all dimensions must be non-negative integers.",
+        f"{shape=}, but all dimensions must be non-negative integers.",
     )
 
     if elements is None:
@@ -812,7 +797,7 @@ def indices(
     check_type(tuple, shape, "shape")
     check_argument(
         all(isinstance(x, int) and x >= 0 for x in shape),
-        f"shape={shape!r}, but all dimensions must be non-negative integers.",
+        f"{shape=}, but all dimensions must be non-negative integers.",
     )
     check_type(bool, allow_newaxis, "allow_newaxis")
     check_type(bool, allow_ellipsis, "allow_ellipsis")
@@ -906,7 +891,7 @@ def make_strategies_namespace(
         check_argument(
             isinstance(xp.__array_api_version__, str)
             and xp.__array_api_version__ in RELEASED_VERSIONS,
-            f"xp.__array_api_version__={xp.__array_api_version__!r}, but it must "
+            f"{xp.__array_api_version__=}, but it must "
             f"be a valid version string {RELEASED_VERSIONS}. {not_available_msg}",
         )
         api_version = xp.__array_api_version__
@@ -914,7 +899,7 @@ def make_strategies_namespace(
     else:
         check_argument(
             isinstance(api_version, str) and api_version in NOMINAL_VERSIONS,
-            f"api_version={api_version!r}, but it must be None, or a valid version "
+            f"{api_version=}, but it must be None, or a valid version "
             f"string in {RELEASED_VERSIONS}. {not_available_msg}",
         )
         inferred_version = False
@@ -925,6 +910,7 @@ def make_strategies_namespace(
         warn(
             f"Could not determine whether module {xp.__name__} is an Array API library",
             HypothesisWarning,
+            stacklevel=2,
         )
 
     try:
@@ -945,10 +931,10 @@ def make_strategies_namespace(
         allow_subnormal: Optional[bool] = None,
         exclude_min: Optional[bool] = None,
         exclude_max: Optional[bool] = None,
-    ) -> st.SearchStrategy[Union[bool, int, float]]:
+    ) -> st.SearchStrategy[Union[bool, int, float, complex]]:
         return _from_dtype(
             xp,
-            api_version,  # type: ignore[arg-type]
+            api_version,
             dtype,
             min_value=min_value,
             max_value=max_value,
@@ -972,7 +958,7 @@ def make_strategies_namespace(
     ) -> st.SearchStrategy:
         return _arrays(
             xp,
-            api_version,  # type: ignore[arg-type]
+            api_version,
             dtype,
             shape,
             elements=elements,
@@ -982,7 +968,7 @@ def make_strategies_namespace(
 
     @defines_strategy()
     def scalar_dtypes() -> st.SearchStrategy[DataType]:
-        return _scalar_dtypes(xp, api_version)  # type: ignore[arg-type]
+        return _scalar_dtypes(xp, api_version)
 
     @defines_strategy()
     def boolean_dtypes() -> st.SearchStrategy[DataType]:
@@ -994,7 +980,7 @@ def make_strategies_namespace(
 
     @defines_strategy()
     def numeric_dtypes() -> st.SearchStrategy[DataType]:
-        return _numeric_dtypes(xp, api_version)  # type: ignore[arg-type]
+        return _numeric_dtypes(xp, api_version)
 
     @defines_strategy()
     def integer_dtypes(
@@ -1027,7 +1013,7 @@ def make_strategies_namespace(
     class StrategiesNamespace(SimpleNamespace):
         def __init__(self, **kwargs):
             for attr in ["name", "api_version"]:
-                if attr not in kwargs.keys():
+                if attr not in kwargs:
                     raise ValueError(f"'{attr}' kwarg required")
             super().__init__(**kwargs)
 
@@ -1128,7 +1114,7 @@ if np is not None:
 
     mock_xp = SimpleNamespace(
         __name__="mock",
-        __array_api_version__="2021.12",
+        __array_api_version__="2022.12",
         # Data types
         int8=np.int8,
         int16=np.int16,
@@ -1154,10 +1140,8 @@ if np is not None:
         arange=np.arange,
         asarray=np.asarray,
         empty=np.empty,
-        full=np.full,
         zeros=np.zeros,
         ones=np.ones,
-        linspace=np.linspace,
         # Manipulation functions
         reshape=np.reshape,
         # Element-wise functions
