@@ -1,36 +1,34 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
+import codecs
 import gzip
 import json
 import os
 import sys
 import tempfile
 import unicodedata
+from functools import lru_cache
 from typing import Dict, Tuple
 
 from hypothesis.configuration import mkdir_p, storage_directory
 from hypothesis.errors import InvalidArgument
+from hypothesis.internal.intervalsets import IntervalSet
 
 intervals = Tuple[Tuple[int, int], ...]
-cache_type = Dict[Tuple[Tuple[str, ...], int, int, intervals], intervals]
+cache_type = Dict[Tuple[Tuple[str, ...], int, int, intervals], IntervalSet]
 
 
-def charmap_file():
+def charmap_file(fname="charmap"):
     return storage_directory(
-        "unicode_data", unicodedata.unidata_version, "charmap.json.gz"
+        "unicode_data", unicodedata.unidata_version, f"{fname}.json.gz"
     )
 
 
@@ -100,6 +98,43 @@ def charmap():
     return _charmap
 
 
+@lru_cache(maxsize=None)
+def intervals_from_codec(codec_name: str) -> IntervalSet:  # pragma: no cover
+    """Return an IntervalSet of characters which are part of this codec."""
+    assert codec_name == codecs.lookup(codec_name).name
+    fname = charmap_file(f"codec-{codec_name}")
+    try:
+        with gzip.GzipFile(fname) as gzf:
+            encodable_intervals = json.load(gzf)
+
+    except Exception:
+        # This loop is kinda slow, but hopefully we don't need to do it very often!
+        encodable_intervals = []
+        for i in range(sys.maxunicode + 1):
+            try:
+                chr(i).encode(codec_name)
+            except Exception:  # usually _but not always_ UnicodeEncodeError
+                pass
+            else:
+                encodable_intervals.append((i, i))
+
+    res = IntervalSet(encodable_intervals)
+    res = res.union(res)
+    try:
+        # Write the Unicode table atomically
+        tmpdir = storage_directory("tmp")
+        mkdir_p(tmpdir)
+        fd, tmpfile = tempfile.mkstemp(dir=tmpdir)
+        os.close(fd)
+        # Explicitly set the mtime to get reproducible output
+        with gzip.GzipFile(tmpfile, "wb", mtime=1) as o:
+            o.write(json.dumps(res.intervals).encode())
+        os.renames(tmpfile, fname)
+    except Exception:
+        pass
+    return res
+
+
 _categories = None
 
 
@@ -146,131 +181,11 @@ def as_general_categories(cats, name="cats"):
             out.update(x for x in cs if x.startswith(c))
         elif c not in cs:
             raise InvalidArgument(
-                "In %s=%r, %r is not a valid Unicode category." % (name, cats, c)
+                f"In {name}={cats!r}, {c!r} is not a valid Unicode category."
             )
     return tuple(c for c in cs if c in out)
 
 
-def _union_intervals(x, y):
-    """Merge two sequences of intervals into a single tuple of intervals.
-
-    Any integer bounded by `x` or `y` is also bounded by the result.
-
-    >>> _union_intervals([(3, 10)], [(1, 2), (5, 17)])
-    ((1, 17),)
-    """
-    if not x:
-        return tuple((u, v) for u, v in y)
-    if not y:
-        return tuple((u, v) for u, v in x)
-    intervals = sorted(x + y, reverse=True)
-    result = [intervals.pop()]
-    while intervals:
-        # 1. intervals is in descending order
-        # 2. pop() takes from the RHS.
-        # 3. (a, b) was popped 1st, then (u, v) was popped 2nd
-        # 4. Therefore: a <= u
-        # 5. We assume that u <= v and a <= b
-        # 6. So we need to handle 2 cases of overlap, and one disjoint case
-        #    |   u--v     |   u----v   |       u--v  |
-        #    |   a----b   |   a--b     |  a--b       |
-        u, v = intervals.pop()
-        a, b = result[-1]
-        if u <= b + 1:
-            # Overlap cases
-            result[-1] = (a, max(v, b))
-        else:
-            # Disjoint case
-            result.append((u, v))
-    return tuple(result)
-
-
-def _subtract_intervals(x, y):
-    """Set difference for lists of intervals. That is, returns a list of
-    intervals that bounds all values bounded by x that are not also bounded by
-    y. x and y are expected to be in sorted order.
-
-    For example _subtract_intervals([(1, 10)], [(2, 3), (9, 15)]) would
-    return [(1, 1), (4, 8)], removing the values 2, 3, 9 and 10 from the
-    interval.
-    """
-    if not y:
-        return tuple(x)
-    x = list(map(list, x))
-    i = 0
-    j = 0
-    result = []
-    while i < len(x) and j < len(y):
-        # Iterate in parallel over x and y. j stays pointing at the smallest
-        # interval in the left hand side that could still overlap with some
-        # element of x at index >= i.
-        # Similarly, i is not incremented until we know that it does not
-        # overlap with any element of y at index >= j.
-
-        xl, xr = x[i]
-        assert xl <= xr
-        yl, yr = y[j]
-        assert yl <= yr
-
-        if yr < xl:
-            # The interval at y[j] is strictly to the left of the interval at
-            # x[i], so will not overlap with it or any later interval of x.
-            j += 1
-        elif yl > xr:
-            # The interval at y[j] is strictly to the right of the interval at
-            # x[i], so all of x[i] goes into the result as no further intervals
-            # in y will intersect it.
-            result.append(x[i])
-            i += 1
-        elif yl <= xl:
-            if yr >= xr:
-                # x[i] is contained entirely in y[j], so we just skip over it
-                # without adding it to the result.
-                i += 1
-            else:
-                # The beginning of x[i] is contained in y[j], so we update the
-                # left endpoint of x[i] to remove this, and increment j as we
-                # now have moved past it. Note that this is not added to the
-                # result as is, as more intervals from y may intersect it so it
-                # may need updating further.
-                x[i][0] = yr + 1
-                j += 1
-        else:
-            # yl > xl, so the left hand part of x[i] is not contained in y[j],
-            # so there are some values we should add to the result.
-            result.append((xl, yl - 1))
-
-            if yr + 1 <= xr:
-                # If y[j] finishes before x[i] does, there may be some values
-                # in x[i] left that should go in the result (or they may be
-                # removed by a later interval in y), so we update x[i] to
-                # reflect that and increment j because it no longer overlaps
-                # with any remaining element of x.
-                x[i][0] = yr + 1
-                j += 1
-            else:
-                # Every element of x[i] other than the initial part we have
-                # already added is contained in y[j], so we move to the next
-                # interval.
-                i += 1
-    # Any remaining intervals in x do not overlap with any of y, as if they did
-    # we would not have incremented j to the end, so can be added to the result
-    # as they are.
-    result.extend(x[i:])
-    return tuple(map(tuple, result))
-
-
-def _intervals(s):
-    """Return a tuple of intervals, covering the codepoints of characters in
-    `s`.
-
-    >>> _intervals('abcdef0123456789')
-    ((48, 57), (97, 102))
-    """
-    intervals = tuple((ord(c), ord(c)) for c in sorted(s))
-    return _union_intervals(intervals, intervals)
-
-
 category_index_cache = {(): ()}
 
 
@@ -311,14 +226,17 @@ def _query_for_key(key):
         pass
     assert key
     if set(key) == set(categories()):
-        result = ((0, sys.maxunicode),)
+        result = IntervalSet([(0, sys.maxunicode)])
     else:
-        result = _union_intervals(_query_for_key(key[:-1]), charmap()[key[-1]])
-    category_index_cache[key] = result
-    return result
+        result = IntervalSet(_query_for_key(key[:-1])).union(
+            IntervalSet(charmap()[key[-1]])
+        )
+    assert isinstance(result, IntervalSet)
+    category_index_cache[key] = result.intervals
+    return result.intervals
 
 
-limited_category_index_cache = {}  # type: cache_type
+limited_category_index_cache: cache_type = {}
 
 
 def query(
@@ -330,7 +248,7 @@ def query(
     exclude_characters="",
 ):
     """Return a tuple of intervals covering the codepoints for all characters
-    that meet the critera (min_codepoint <= codepoint(c) <= max_codepoint and
+    that meet the criteria (min_codepoint <= codepoint(c) <= max_codepoint and
     any(cat in include_categories for cat in categories(c)) and all(cat not in
     exclude_categories for cat in categories(c)) or (c in include_characters)
 
@@ -341,7 +259,7 @@ def query(
     >>> query(min_codepoint=0, max_codepoint=128, include_categories=['Lu'])
     ((65, 90),)
     >>> query(min_codepoint=0, max_codepoint=128, include_categories=['Lu'],
-    ...       include_characters=u'☃')
+    ...       include_characters='☃')
     ((65, 90), (9731, 9731))
     """
     if min_codepoint is None:
@@ -349,14 +267,14 @@ def query(
     if max_codepoint is None:
         max_codepoint = sys.maxunicode
     catkey = _category_key(exclude_categories, include_categories)
-    character_intervals = _intervals(include_characters or "")
-    exclude_intervals = _intervals(exclude_characters or "")
+    character_intervals = IntervalSet.from_string(include_characters or "")
+    exclude_intervals = IntervalSet.from_string(exclude_characters or "")
     qkey = (
         catkey,
         min_codepoint,
         max_codepoint,
-        character_intervals,
-        exclude_intervals,
+        character_intervals.intervals,
+        exclude_intervals.intervals,
     )
     try:
         return limited_category_index_cache[qkey]
@@ -367,8 +285,6 @@ def query(
     for u, v in base:
         if v >= min_codepoint and u <= max_codepoint:
             result.append((max(u, min_codepoint), min(v, max_codepoint)))
-    result = tuple(result)
-    result = _union_intervals(result, character_intervals)
-    result = _subtract_intervals(result, exclude_intervals)
+    result = (IntervalSet(result) | character_intervals) - exclude_intervals
     limited_category_index_cache[qkey] = result
     return result
